#!/usr/bin/perl
###############################################################################
#    Webmin Sysstats - mem-lib.pl
#
#    Webmin Sysstats Module ver 0.6
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: dbouius@users.sourceforge.net
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

sub install_module {
  local ($minfo, $defaults) = ($_[0], $_[1]);

  local (%my_config);

  if (!-r $minfo->{'dbtables'} . "mem.rrd") {
    local ($cmd);

    $cmd = $config{'rrdtool'};
    $cmd .= " create " . $minfo->{'dbtables'} . "mem.rrd";
    $cmd .= " --start 0 --step 60";
    $cmd .= " DS:mtotal:GAUGE:100:U:U";
    $cmd .= " DS:mused:GAUGE:100:U:U";
    $cmd .= " DS:stotal:GAUGE:100:U:U";
    $cmd .= " DS:sused:GAUGE:100:U:U";
    $cmd .= " RRA:AVERAGE:0.5:1:70";
    $cmd .= " RRA:AVERAGE:0.5:2:750";
    $cmd .= " RRA:AVERAGE:0.5:15:650";
    $cmd .= " RRA:AVERAGE:0.5:50:650";

    `$cmd`;
  }

  $my_config{'sample_rate'} = '1';

  $my_config{'mem'} = join(',',
			   $defaults->{'real_colour'},
			   $defaults->{'swap_colour'}
			  );

  &write_config($minfo, \%my_config, $defaults);
  &install_script($minfo, 'mem.sh');

  return 1;
}

sub remove_module {
}

sub mod_check_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  return $my_config->{'runstop'};
}

sub mod_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'runstop'} = $my_config->{'runstop'} ? '1' : '0';
  &write_config($minfo, $my_config, $defaults);
}

sub mod_check_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if (-r $minfo->{'dbtables'} . "../mem/mem.rrd") {
    return 1;
  }
  return 0;
}

sub mod_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%old_config);

  if (-r $minfo->{'dbtables'} . "../mem/mem.rrd") {
    my $cmd = "cp -f " . $minfo->{'dbtables'} . "../mem/mem.rrd " . $minfo->{'dbtables'};

    system($cmd);

    if (-r $minfo->{'config_dir'} . "../mem/module.cfg" ) {
      if (!&read_file($minfo->{'config_dir'} . "../mem/module.cfg", \%old_config)) {
	&error(&text('ecannot_readconfig', $minfo->{'config_dir'} . "../mem/module.cfg"));
      }

      $my_config->{'mem'} = $old_config{'mem'};

      &write_config($minfo, $my_config, $defaults);
    }
  }
}

sub graph_data {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $rrdtool = $config{'rrdtool'};
  $sizex = $config{'graph_sizex'};
  $sizey = $config{'graph_sizey'};

  $rrd_detals  = " -u 100 --rigid -v '% utilization'";

  ($real_colour, $swap_colour) = split(',', $my_config->{'mem'});

  $rrd_detals  = " -l 0 -u 100 -v '% utilization'";
  $rrd_detals .= " DEF:mtotal=" . $minfo->{'dbtables'} . "mem.rrd:mtotal:AVERAGE";
  $rrd_detals .= " DEF:mused=" . $minfo->{'dbtables'} . "mem.rrd:mused:AVERAGE";
  $rrd_detals .= " DEF:stotal=" . $minfo->{'dbtables'} . "mem.rrd:stotal:AVERAGE";
  $rrd_detals .= " DEF:sused=" . $minfo->{'dbtables'} . "mem.rrd:sused:AVERAGE";
  $rrd_detals .= " CDEF:mmed=mused,100,*,mtotal,/";
  $rrd_detals .= " CDEF:smed=sused,100,*,stotal,/";
  $rrd_detals .= " COMMENT:'Type        Max      Min  Average\\n'";
  $rrd_detals .= " LINE2:mmed#" . $real_colour . ":'Memory'";
  $rrd_detals .= " GPRINT:mmed:MAX:'%6.2lf'";
  $rrd_detals .= " GPRINT:mmed:MIN:'%6.2lf'";
  $rrd_detals .= " GPRINT:mmed:AVERAGE:'%6.2lf\\n'";
  $rrd_detals .= " LINE2:smed#" . $swap_colour . ":'Swap  '";
  $rrd_detals .= " GPRINT:smed:MAX:'%6.2lf'";
  $rrd_detals .= " GPRINT:smed:MIN:'%6.2lf'";
  $rrd_detals .= " GPRINT:smed:AVERAGE:'%6.2lf\\n'";

  foreach $graph ('hour', 'day', 'week', 'month') {
    if ($graph eq 'hour') {
      $time = "-3600";
      $title = "'The last hour'";
    } elsif ($graph eq 'day') {
      $time = "-86400";
      $title = "'The last day'";
    } elsif ($graph eq 'week') {
      $time = "-604800";
      $title = "'The last week'";
    } elsif ($graph eq 'month') {
      $time = "-18748800";
      $title = "'The last month'";
    }

    $common  = $rrdtool . " graph " . $minfo->{'tmpgname'} . "_" . $graph . ".png";
    $common .= " --start " . $time . " --end -60";
    $common .= " -h " . $sizey . " -w " . $sizex;
    $common .= " -a PNG --title=" . $title;

    $sig = " COMMENT:'\\n' COMMENT:\"Created: `date`\"";

    system($common . $rrd_detals . $sig);
  }
}

sub mod_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  ($real_colour, $swap_colour) = split(',', $my_config->{'mem'});
  print "<table width=100%><tr><td><b>";
  print  $text{'module_subtitle'};
  print "</b></td></tr></table>\n";

  print "<form action=save_config.cgi>\n";
  print  "<input type=hidden name=idx value='$in{'idx'}'>\n";

  print   "<table width=100%>\n";
  print    "<tr $cb>";
  print     "<td>";
  print      "<table width=100%>\n";

  print       "<tr>";
  print        "<td width=15%>";
  print         "<b>" . $text{'module_word_real'} . "</b>";
  print        "</td>";

  print        "<td width=35%>";
  &option_colourselect('real', $real_colour,
		       $defaults->{'real_colour'}
		      );
  print        "</td>";
  print        "<td width=15%>";
  print         "<b>" . $text{'module_word_swap'} . "</b>";
  print        "</td>";

  print        "<td width=35%>";
  &option_colourselect('swap', $swap_colour,
		       $defaults->{'swap_colour'}
		      );
  print        "</td>";
  print       "</tr>";
  print      "</table>";
  print     "</td>";
  print    "</tr>";
  print   "</table>\n";
  print  "<input type=submit value='" . $text{'word_save'} . "'>";
  print "</form>\n";
}

sub mod_config_save {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'mem'} = join ( ',',
			     ($in{'real2_def'} ? $in{'real2_def'} : $in{'real1_def'} eq 'default'
			      ? $defaults->{'real_colour'} : $in{'real1_def'}),
			     ($in{'swap2_def'} ? $in{'swap2_def'} : $in{'swap1_def'} eq 'default'
			      ? $defaults->{'swap_colour'} : $in{'swap1_def'})
			   );

  &write_config($minfo, $my_config, $defaults);
}

sub write_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%new_config, %sh_config);

  $new_config{'sample_rate'} = $my_config->{'sample_rate'};
  $sh_config{'sample_rate'} = $my_config->{'sample_rate'};
  $new_config{'runstop'} = $my_config->{'runstop'};
  $new_config{'mem'} = $my_config->{'mem'};

  if (!$new_config{'runstop'}) {
    $sh_config{'mem'} = 'mem';
  }

  unlink ($minfo->{'config_file'});
  unlink ($minfo->{'config_dir'} . 'mem.cfg');

  &write_file($minfo->{'config_file'}, \%new_config);
  &write_file($minfo->{'config_dir'} . 'mem.cfg', \%sh_config);
}

return 1;

